<?php
namespace NsFWK;

class ClsBllRedis {

    protected $_client;
    private $_params = [];

    public function __get($name){
        return $this->$name;
    }

    private function setDefaultValues(){
        $this->_params = [
            'scheme'=>DSN_REDIS_SCHEME,
            'host'=>DSN_REDIS_HOST,
            'port'=>DSN_REDIS_PORT,
            'database' => DSN_REDIS_DATABASE,
            'username' => DSN_REDIS_USERNAME,
            'password' => DSN_REDIS_PASSWORD
        ];
    }

    public function __construct($parameters = [], $options = []){
        if(empty($parameters)){
            $this->setDefaultValues();
        }else{
            $this->_params = $parameters;
        }
        $this->_client = new \Predis\Client($this->_params, $options);        
    }

    public function DeleteKey($key){
        $rslt = $this->_client->del($key);
        return $rslt;
    }

    public function SetValue($key, $value){
        $strData = is_array($value) ? json_encode($value) : $value;
        $ok = $this->_client->set($key, $strData);
        if ($ok) {
            return true;
        } else {
            return false;
        }
    }

    public function List($pattern){
        $arrKeys  = [];
        
        foreach (new \Predis\Collection\Iterator\Keyspace($this->_client, $pattern) as $key) {
            $arrKeys[] = $key;
        }        
        return $arrKeys;
    }

    public function GetValue($key)
    {
        $strValue = $this->_client->get($key);
        $arrData = json_decode($strValue,true);
        if(json_last_error()==JSON_ERROR_NONE){
            return $arrData;
        }
        return $strValue;
    }

    public function Exists($key){
        return $this->_client->exists($key);
    }

    public function SetExpiry($key, $intSeconds, $value){
        if($this->_client->exists($key)){
            $intTime2Live = $this->_client->ttl($key);
            //$intNewExpiry = $intSeconds + $intTime2Live;
            $intNewExpiry = $intSeconds;
            $this->_client->expire($key, $intNewExpiry);
        }else{
            if(is_array($value)){
                $strValue = json_encode($value);
            }else{
                $strValue = $value;
            }
            $this->_client->setex($key, $intSeconds, $strValue);
        }
    }

    public function GetExpiry($key){
        if($this->_client->exists($key)){
            $ttl = $this->_client->ttl($key);
            return $ttl;
        }else{
            return false;
        }

    }

}

