<?php
namespace NsFWK;

class ClsBllVideoDownload 
{

    public const EVENT_VERBOSE = 'EventVerbose';
    public const EVENT_PROGRESS = 'EventProgress';
    public const EXIPRY_SECONDS = 7200;
    private const DOWNLOAD_CHUCK_SIZE = 10485760;

    private \NsFWK\ClsBllRedis $_objRedis;
    private \NsFWK\ClsBllEventEmitter $_emitter;

    private int $_matchId, $_streamId, $_qualityId, $_userId;
    private string $_matchName;
    private string $_tmpMatchFolder;
    private string $_serverIpAddress;
    private array $_arrMeta;

    private $_globalProgress;
    private $_globalStep;
    private $_globalStart;


    public function __construct($intUserID, $intMatchID, $strMatchName, $intStreamID, $intQualityID){
        $params = [
            'scheme'=>DSN_REDIS_SCHEME,
            'host'=>DSN_REDIS_HOST,
            'port'=>DSN_REDIS_PORT,
            'database' => DNS_REDIS_DATABASE_VIDEO,
            'username' => DSN_REDIS_USERNAME,
            'password' => DSN_REDIS_PASSWORD
        ];
        $this->_emitter = new \NsFWK\ClsBllEventEmitter();
        $this->_objRedis = new \NsFWK\ClsBllRedis($params);
        $this->_userId = $intUserID;
        $this->_matchId = $intMatchID;
        $this->_streamId = $intStreamID;
        $this->_qualityId = $intQualityID;
        $this->_serverIpAddress = SERVER_IP_ADDRESS;
        $this->_matchName = $strMatchName;


        $this->_tmpMatchFolder = LOCAL__UPLOAD . "/matches/{$this->_matchId}/";
        \NsFWK\ClsHlpHelper::CreateFolder($this->_tmpMatchFolder);
    }

    public function OnEventFire($event, callable $listener) {
        $this->_emitter->on($event, $listener);
    }

    private function checkIsInDownload($strRedisID){
        if($this->_objRedis->Exists($strRedisID)===false){
            return false;
        }

        $objValue = $this->_objRedis->GetValue($strRedisID);
        if(isset($objValue['status']) && $objValue['status']=='downloading'){
            return true;
        }
        return false;
    }

    public function Download($arrVideos, $strTitle){
        $arrOutputVideos = [];
        $intStep = 0;
        $decStep = 1/count($arrVideos);
        $this->_globalStep = $decStep;
        $this->_globalStart = 0;
        foreach($arrVideos as $intHalf=>$strVideo){           
            $strDstPath = $this->_tmpMatchFolder . $this->_matchId . "_" . $intHalf . ".mp4";
            $arrOutputVideos[] = $strDstPath;
            $arrVideosLocal[$intHalf] = $strDstPath;

            $this->_emitter->emit(self::EVENT_VERBOSE, "File found: $strDstPath");

            $strRedisID = sprintf("%s-%s-%s", $this->_serverIpAddress, $this->_matchId, $intHalf);

            $boolResult = $this->checkIsInDownload($strRedisID);
            while($boolResult){
                $boolResult = $this->checkIsInDownload($strRedisID);
                $this->_emitter->emit(self::EVENT_VERBOSE, "Waiting for download");
                sleep(10);
            }            
            RepeatTask:
            if(file_exists($strDstPath)){
                $objRedisValue = $this->_objRedis->GetValue($strRedisID);
                $decStoredSize = $objRedisValue['size'];
                $decFileSize = filesize($strDstPath);
                if($decFileSize===$decStoredSize){
                    $boolExists = true;
                    $objRedisValue = [
                        'matchId'=>$this->_matchId, 
                        'half'=>$intHalf,
                        'streamId'=>$this->_streamId, 
                        'qualityId'=>$this->_qualityId, 
                        'filename'=>$strDstPath, 
                        'ip'=>$this->_serverIpAddress,
                        'downloaded'=>$boolExists,
                        'size'=>$decFileSize,
                        'status'=>'downloaded'
                    ];    
                }else{
                    sleep(10);
                    goto RepeatTask;
                }
                $this->_objRedis->SetExpiry($strRedisID, self::EXIPRY_SECONDS, $objRedisValue);

                $this->_emitter->emit(self::EVENT_VERBOSE, "File {$strDstPath} Exists size: {$decFileSize}");
            }else{
                $decFileSize = 0;
                $boolExists = false;
                $objRedisValue = [
                    'matchId'=>$this->_matchId, 
                    'half'=>$intHalf,
                    'streamId'=>$this->_streamId, 
                    'qualityId'=>$this->_qualityId, 
                    'filename'=>$strDstPath, 
                    'ip'=>$this->_serverIpAddress,
                    'downloaded'=>$boolExists,
                    'size'=>$decFileSize,
                    'status'=>'downloading'
                ];
                $this->_emitter->emit(self::EVENT_VERBOSE, "File {$strDstPath} NOT EXIST, we will downlaod it.");
                $this->_emitter->emit(self::EVENT_PROGRESS,$this->_globalStart, $strTitle, "File {$strDstPath} NOT EXIST, we will downlaod it");                
                $strMessage = "Downloading {$this->_matchName} H{$intHalf}";
                $this->_downloadFile($strVideo,$strDstPath, $strTitle, $strMessage);
                $this->_emitter->emit(self::EVENT_PROGRESS,$this->_globalStart, $strTitle, "File {$strDstPath} Downloaded it");                
                $boolExists = true;
                $decFileSize = filesize($strDstPath);
                $decStatus = 'downloaded';

                $objRedisValue = [
                    'matchId'=>$this->_matchId, 
                    'half'=>$intHalf,
                    'streamId'=>$this->_streamId, 
                    'qualityId'=>$this->_qualityId, 
                    'filename'=>$strDstPath, 
                    'ip'=>$this->_serverIpAddress,
                    'downloaded'=>$boolExists,
                    'size'=>$decFileSize,
                    'status'=>$decStatus
                ];
                $this->_objRedis->SetValue($strRedisID, $objRedisValue);
                $this->_objRedis->SetExpiry($strRedisID, self::EXIPRY_SECONDS, $objRedisValue);
            }
            $strMessage = "{$this->_matchName} H{$intHalf} Downloaded";
            $this->_globalStart+=$decStep;
            $this->_emitter->emit(self::EVENT_PROGRESS,$this->_globalStart, $strTitle, $strMessage);
        }
        return $arrOutputVideos;
    }
    private function _downloadFile($url, $outputPath, $strTitle, $strMessage) {
        $client = new \GuzzleHttp\Client();
        $fileHandle = fopen($outputPath, 'w');
        $response = $client->head($url);
        $fileSize = $response->getHeader('Content-Length')[0];
        $chunkSize = 10485760; // 10 MB
    
        $totalDownloaded = 0;
        $lastProgressUpdate = 0;
    
        for ($start = 0; $start < $fileSize; $start += $chunkSize) {
            $end = min($start + $chunkSize - 1, $fileSize - 1);
            $response = $client->get($url, [
                'headers' => [
                    'Range' => "bytes=$start-$end"
                ],
                'stream' => true,
            ]);
    
            $body = $response->getBody();
            while (!$body->eof()) {
                $chunk = $body->read($chunkSize);
                fwrite($fileHandle, $chunk);
                $totalDownloaded += strlen($chunk);
    
                // Calculate and display progress
                $progress = ($totalDownloaded / $fileSize);
                $this->_globalProgress = $this->_globalStart + ($this->_globalStep * $progress);
    
                // Only emit progress if it has changed by at least 1%
                if (floor($progress * 100) > $lastProgressUpdate) {
                    $lastProgressUpdate = floor($progress * 100);
                    $this->_emitter->emit(self::EVENT_PROGRESS, $this->_globalProgress, $strTitle, $strMessage);
                }
            }
        }
        fclose($fileHandle);
    }

}