<?php
namespace NsFWK;

abstract class ClsCtrlCron extends ClsBase {

    public function __construct(array $arrParameters = []) {
        $this->sanitizeParameters($arrParameters);
        $this->_data = $arrParameters;
        $this->logCtrlCall();
    }

    public function DoAction() {
        $fnAction = isset($this->_data['action']) ? "do_" . $this->_data['action'] : 'do_Default';

        if (method_exists($this, $fnAction)) {
            $this->$fnAction();
        } else {
            $this->handleActionNotFound();
        }
    }

    protected function logCtrlCall() {
        $strLog = isset($_SERVER['HTTP_USER_AGENT']) && isset($_SERVER['REMOTE_ADDR'])
            ? "New call by User Agent: [{$_SERVER['HTTP_USER_AGENT']}] - FROM IP Address [{$_SERVER['REMOTE_ADDR']}]"
            : "New call by command line";
        
        \NsFWK\ClsLogger::GetInstance()->LogDebug($strLog);
        \NsFWK\ClsLogger::GetInstance()->LogDebug("Parameters: " . json_encode($this->_data));
    }

    protected function verbose($content) {
        $this->print2Console($content);
        $this->print2Log($content);
        $this->flushOutput();
    }

    protected function print2Console($content) {
        $break = PHP_SAPI === 'cli' ? "\n" : "<br/>";
        if (is_array($content)) {
            print_r($content);
        } else {
            print $content;
        }
        print $break;
    }

    protected function print2Log($content) {
        $objLog = \NsFWK\ClsLogger::GetInstance();
        if (is_array($content)) {
            $content = implode(" ", $content);
        }
        $objLog->LogDebug($content);
    }

    protected function flushOutput() {
        if (ob_get_level()) {
            ob_flush(); // Flush (send) the output buffer
        }
        flush(); // Flush system output buffer
    }

    private function sanitizeParameters(array &$arrParameters) {
        if (isset($arrParameters['params']) && is_array($arrParameters['params'])) {
            $arrParameters['params'] = array_map(function($param) {
                return trim($param, "'");
            }, $arrParameters['params']);
        }
    }

    private function handleActionNotFound() {
        $strLog = "Cron not found.";
        \NsFWK\ClsLogger::GetInstance()->LogDebug($strLog);
        print($strLog);
        return false;
    }
}
