<?php
namespace NsFWK;

class ClsHlpCron
{
    // Configuration variables
    private static $dateFormat = "Y-m-d H:i:s";
    private static $outputDelimiter = "\n";
    private static $cronFileName = 'cron.php';
    private static $logFileExtension = '.out';
    private static $processSearchCommand = "ps aux | grep %s | grep -v grep";
    private static $defaultLocale = 'en_US';
    private static $defaultLoginMethod = 'AMQPLAIN';
    private static $defaultInsist = false;
    private static $defaultLoginResponse = null;
    private static $defaultContext = null;
    private static $defaultKeepAlive = true;
    private static $connectionTimeout = 3.0;
    private static $readTimeout = 3.0;
    private static $heartbeatInterval = 7200;
    private static $qosPrefetchCount = 1;
    private static $sleepDurationMicroseconds = 1000; // 1 millisecond

    public static function PrintMessage($msg, $title) {
        if (!RABBITMQ_DEBUG) {
            return;
        }
        echo "Date: " . date(self::$dateFormat) . " - $title : $msg " . self::$outputDelimiter;
    }

    public static function StartEncodingJob(array $arrData): bool {
        $strJobDetails = implode(',', [
            $arrData['report_progress'],
            $arrData['job_id'],
            $arrData['visibility'],
            $arrData['title']
        ]);

        self::CallCron(LOCAL__PHP__CLI, LOCAL__ROOT__PATH, $arrData['module'], $arrData['cron'], $arrData['arrParam'], $strJobDetails);
        return true;
    }

    public static function CallCron($strCLIPath, $strCronPath, $strModule, $strCron, array $arrParams, $strJobDetails = ',') {
        $strCronPath .= '/' . self::$cronFileName;
        $strParams = !empty($arrParams) ? "'" . implode("' '", $arrParams) . "'" : '';

        $cmd = sprintf(
            "%s -q %s '%s' '%s' '%s' %s",
            $strCLIPath, $strCronPath, $strModule, $strCron, $strJobDetails, $strParams
        );

        $strCronLog = sprintf("%s/%s__%s%s", LOCAL__LOGS, $strCron, date('Y-m-d__H-i-s'), self::$logFileExtension);
        $cmd = "nohup " . $cmd . " > " . $strCronLog . " 2>&1 &";

        return exec($cmd);
    }

    public static function CanStartNewJob($intMaxConcurrency): bool {
        $checkCommand = sprintf(self::$processSearchCommand, escapeshellarg(RABBITMQ_PROCESS_FILE));
        exec($checkCommand, $output, $returnVar);

        return count($output) < $intMaxConcurrency;
    }

    public static function Process($strQueueName, $strExchange, $strRoutingKey, $intMaxConcurrency) {
        try {
            $connection = new \PhpAmqpLib\Connection\AMQPStreamConnection(
                RABBITMQ_HOST,
                RABBITMQ_PORT,
                RABBITMQ_USER,
                RABBITMQ_PASSWORD,
                RABBITMQ_VHOST,
                self::$defaultInsist,
                self::$defaultLoginMethod,
                self::$defaultLoginResponse,
                self::$defaultLocale,
                self::$connectionTimeout,
                self::$readTimeout,
                self::$defaultContext,
                self::$defaultKeepAlive,
                self::$heartbeatInterval
            );
            $channel = $connection->channel();

            $channel->exchange_declare($strExchange, 'direct', false, true, false);
            $channel->queue_declare($strQueueName, false, true, false, false);
            $channel->queue_bind($strQueueName, $strExchange, $strRoutingKey);

            $callback = function (\PhpAmqpLib\Message\AMQPMessage $msg) use ($channel, $intMaxConcurrency) {
                try {
                    $arrData = json_decode($msg->body, true);
                    if (\NsFWK\ClsHlpCron::CanStartNewJob($intMaxConcurrency)) {
                        if (\NsFWK\ClsHlpCron::StartEncodingJob($arrData)) {
                            self::PrintMessage("Before Ack", "info");
                            self::PrintMessage("Delivery Tag: " . json_encode($msg->get('delivery_tag'), true), "info");
                            $channel->basic_ack($msg->get('delivery_tag'));
                            self::PrintMessage("After Ack", "info");
                            usleep(self::$sleepDurationMicroseconds);
                        } else {
                            $channel->basic_nack($msg->get('delivery_tag'), false, true);
                            self::PrintMessage("Job Did not start, Reject and Requeue", "info");
                        }
                    } else {
                        $channel->basic_nack($msg->get('delivery_tag'), false, true);
                        self::PrintMessage("Another job is working", "info");
                    }
                } catch (\Exception $e) {
                    error_log($e->getMessage());
                    self::PrintMessage($e->getMessage(), "Exception");
                    $channel->basic_nack($msg->get('delivery_tag'), false, true);
                }
            };

            $channel->basic_qos(null, self::$qosPrefetchCount, null);
            $channel->basic_consume($strQueueName, \NsFWK\ClsHlpHelper::GetServerName(), false, false, false, false, $callback);

            while ($channel->is_consuming()) {
                $channel->wait(null, false, self::$heartbeatInterval);
            }

            $channel->close();
            $connection->close();
        } catch (\Exception $e) {
            error_log($e->getMessage());

            if (isset($channel) && $channel->is_open()) {
                $channel->close();
            }
            if (isset($connection) && $connection->isConnected()) {
                $connection->close();
            }

            throw $e;
        }
    }
}
