<?php
/* 
* This class has been based on the orginal code by Kenneth Katzgaru.
* I'm keeping the original comment for copyrights
* Mostafa Zakaria
*  
* Finally, A light, permissions-checking logging class. 
* 
* Author    : Kenneth Katzgrau < katzgrau@gmail.com >
* Date      : July 26, 2008
* Comments  : Originally written for use with wpSearch
* Website   : http://codefury.net
* Version   : 1.0
*
* Usage: 
*        $log = new ClsLogger("log.txt", self::LEVEL_INFO);
*        $log->LogInfo("Returned a million search results");    //Prints to the log file
*        $log->LogFATAL("Oh dear.");                //Prints to the log file
*        $log->LogDebug("x = 5");                    //Prints nothing due to level setting
*/
namespace NsFWK;

class ClsLoggerFile extends \NsFWK\ClsLog {
    
    // File managment
    const LOG_OPEN      = 1;
    const OPEN_FAILED   = 2;
    const LOG_CLOSED    = 3;

    protected $_data = array();

    public function __set($name, $value){
        return false;
    }

    public function __get($name){
        if(array_key_exists($name, $this->_data)){
            return $this->_data[$name];
        }

        return null;
    }
    
    public function __isset($name){
        $value = $this->__get($name);
        return isset($value);
    }

    public function __construct($arrConfig){
        $this->_data = [
            'intLogLevel' => $arrConfig['intLogLevel'],
            'strRunMode' => $arrConfig['strRunMode'],
            'strVersion' => $arrConfig['strVersion'],
            'strEntryPoint' => $arrConfig['strEntryPoint'],
            'strAccountName' => $arrConfig['strAccountName'],
            'strLogPath' => $arrConfig['strLogPath'],
            'arrMessageQueue' => [],
            'intFileStatus' => self::LOG_OPEN,
            'strLogFile' => '',
            'fileHandle' => null
        ];
        $strFileName = 'log_' . date('Ymd') . '.log';
        $strLogFile = $this->_data['strLogPath'] . "/$strFileName";
        $this->_data['strLogFile'] = $strLogFile;
        
        if(!file_exists($this->_data['strLogPath'])){
            $ok = mkdir($this->_data['strLogPath']);
        }

        if(file_exists($this->_data['strLogFile'])){
            if (!is_writable($this->_data['strLogFile'])){
                $this->_data['intFileStatus'] = self::OPEN_FAILED;
                $this->_data['arrMessageQueue'][] = "The file exists, but could not be opened for writing. Check that appropriate permissions have been set.";
                return;
            }
        }

        if($this->_data['fileHandle'] = fopen($this->_data['strLogFile'], "a")){
            $this->_data['intFileStatus'] = self::LOG_OPEN;
            $this->_data['arrMessageQueue'][] = "The log file was opened successfully.";
        }else{
            $this->_data['intFileStatus'] = self::OPEN_FAILED;
            $this->_data['arrMessageQueue'][] = "The file could not be opened. Check permissions.";
        }

        return $this;
    }

    public function __destruct(){
        if($this->_data['fileHandle']){
            fclose($this->_data['fileHandle']);
        }
    }


    private function getTimeLine( $level ){
        $time = date('Y-m-d H:i:s');
        switch( $level ){
            case self::LEVEL_INFO:
                return "$time - INFO  -";
            case self::LEVEL_WARN:
                return "$time - WARN  -";                
            case self::LEVEL_DEBUG:
                return "$time - DEBUG -";                
            case self::LEVEL_ERROR:
                return "$time - ERROR -";
            case self::LEVEL_FATAL:
                return "$time - FATAL -";
            default:
                return "$time - LOG   -";
        }
    }


    public function LogDebug($strLine, $arrAdditionalField = []){
        if (self::LEVEL_DEBUG >= $this->_data['intLogLevel']) {
            $this->log(self::LEVEL_DEBUG, $strLine, $arrAdditionalField);
        }
    }

    public function LogInfo($strLine, $arrAdditionalField = []){
        if (self::LEVEL_INFO >= $this->_data['intLogLevel']) {
            $this->log(self::LEVEL_INFO, $strLine, $arrAdditionalField);
        }
    }

    public function LogWarn($strLine, $arrAdditionalField = []){
        if (self::LEVEL_WARN >= $this->_data['intLogLevel']) {
            $this->log(self::LEVEL_WARN, $strLine, $arrAdditionalField);
        }
    }

    public function LogError($strLine, $arrAdditionalField = []){
        if (self::LEVEL_ERROR >= $this->_data['intLogLevel']) {
            $this->log(self::LEVEL_ERROR, $strLine, $arrAdditionalField);
        }
    }

    public function LogFatal($strLine, $arrAdditionalField = []){
        if (self::LEVEL_FATAL >= $this->_data['intLogLevel']) {
            $this->log(self::LEVEL_FATAL, $strLine, $arrAdditionalField);
        }
    }

    protected function log($intLogLevel, $strLine, $arrAdditionalField = []){
        $strAdditionalField = '';
        if (!empty($arrAdditionalField)) {
            foreach($arrAdditionalField as $key => $value) {
                $strAdditionalField .= ", $key => $value";
            } 
        }
        $status = $this->getTimeLine($intLogLevel);
        $strDetails = $this->_data['strAccountName'].' - '.$this->_data['strRunMode'].' - '.$this->_data['strEntryPoint'].' - '.$this->_data['strVersion'];
        $strLine = "$status $strDetails  $strLine $strAdditionalField";
        $this->_data['arrMessageQueue'][] = $strLine;

        if($this->_data['intFileStatus'] == self::LOG_OPEN && $this->_data['intLogLevel'] != self::LEVEL_OFF){
            if(fwrite($this->_data['fileHandle'] , "$strLine\n") === false) {
                $this->_data['arrMessageQueue'][] = "The file could not be written to. Check that appropriate permissions have been set.";
            }
        }
    }

}