<?php

namespace NsFWK;

use Exception;

class ClsVideo {

    private \FFMpeg\FFMpeg $_ffmpeg;
    
    private $_tempPath;
    private $_jobId;
    private $_tmStartTime;
    private $_title, $_message;

    public function __set($name, $value) {
        $this->$name = $value;
    }

    
    public function __construct($strJobId, $tmStartTime, $strTempPath = ''){
        $this->_ffmpeg = \FFMpeg\FFMpeg::create(
            [
                'ffmpeg.binaries'  => FFMPEG__FFMPEG_BINARIES,
                'ffprobe.binaries' => FFMPEG__FFPROBE_BINARIES,
                'timeout'          => FFMPEG__TIMEOUT, // The timeout for the underlying process
                'ffmpeg.threads'   => FFMPEG__THREADS,   // The number of threads that FFMpeg should use
            ]
        );


        if($strTempPath==''){
            $this->_tempPath = sys_get_temp_dir();
        }else{
            $this->_tempPath = $strTempPath;
        }
        $this->_tmStartTime = $tmStartTime;
        $this->_jobId = $strJobId;


    }

    protected function UpdateRedis($strTitle, $strMessage, $decProgress, $intUserID, $strMsgType='info', $tmStartTime = '', $arrMeta=[]){
        if($tmStartTime!=''){
            $tmEndTime = microtime(true);
            $duration = $tmEndTime - $tmStartTime;
            $minutes = floor($duration / 60);
            $seconds = fmod($duration, 60); 
            $strMessage .= "\n" . sprintf(" - Process duration: %02d m %02d s", $minutes, $seconds);
        }

        $arrJob['id'] = $this->_jobId;
        $arrJob['progress'] = $decProgress;
        $arrJob['message'] = $strMessage;
        $arrJob['type'] = $strMsgType;
        $arrJob['title'] = $strTitle;
        $arrJob['userId'] = $intUserID;
        $arrJob['consumer'] = \NsFWK\ClsHlpHelper::GetServerIP();
        $arrJob['meta'] = $arrMeta;
        $obj = new \NsFWK\ClsBllRedis();
        $obj->SetValue($this->_jobId, $arrJob);

    }

    private function redis($decProgress, $strMsgType = "info", $tmStartTime=''){
        $strMessage = $this->_message;
        if($tmStartTime!=''){
            $tmEndTime = microtime(true);
            $duration = $tmEndTime - $tmStartTime;
            $minutes = floor($duration / 60);
            $seconds = fmod($duration, 60); 
            $strMessage.= "\n" . sprintf(" - Process duration: %02d m %02d s", $minutes, $seconds);
        }
        $arrJob['id'] = $this->_jobId;
        $arrJob['progress'] = $decProgress;
        $arrJob['message'] = $strMessage;
        $arrJob['type'] = $strMsgType;
        $arrJob['title'] = $this->_title;
        $arrJob['consumer'] = \NsFWK\ClsHlpHelper::GetServerIP();


        $obj = new \NsFWK\ClsBllRedis();
        $obj->SetValue($this->_jobId, $arrJob);
    }

    protected function progress($index, $count, $base = 0, $scaleFactor = 1) {
        $decCalc = (($index/$count)+$base) * $scaleFactor;
        $decProgress = round($decCalc, 2);
        $this->redis($decProgress, "info", $this->_tmStartTime);
    }

    protected function verbose($content){
        $objLog = \NsFWK\ClsLogger::GetInstance();
        if (is_array($content)){
            $content = implode(" ", $content);
        }else{
            //print $content;
        }
        $content = "Time: " . date("Y-m-d H:i:s") . " - ". $content ;
        $objLog->LogDebug($content);
    }    

    /**
     * Undocumented function
     *
     * @param [type] $url
     * @param [type] $outputPath
     * @param integer $chunkSize
     * @param integer $base: This is for the progress, this is the start of the progress
     * @param integer $scaleFactor: this is the stop of the progress, where the progress value will be multiplied by.
     * @return void
     */
    public function DownloadFile($url, $outputPath, $strTitle, $strMessage, $intUserID, $chunkSize = 10485760, $base = 0, $scaleFactor = 1, $arrMeta = []) {
        $client = new \GuzzleHttp\Client();
        $fileHandle = fopen($outputPath, 'w');
        $response = $client->head($url);
        $fileSize = $response->getHeader('Content-Length')[0];

        for ($start = 0; $start < $fileSize; $start += $chunkSize) {
            $end = min($start + $chunkSize - 1, $fileSize - 1);
            $response = $client->get($url, [
                'headers' => [
                    'Range' => "bytes=$start-$end"
                ],
                'stream' => true,
            ]);

            $body = $response->getBody();
            while (!$body->eof()) {
                fwrite($fileHandle, $body->read($chunkSize));
            }

            $fileProgress = ($end + 1) / $fileSize; // Progress of this file as a percentage (0-1)
            print "File Progress: $fileProgress, Base: $base \n";
            $scaledProgress = $base + ($fileProgress * $scaleFactor); // Scaled progress within the download phase
                        
            $this->UpdateRedis($strTitle, $strMessage, $scaledProgress, $intUserID, 'info', $this->_tmStartTime, $arrMeta);
            
            // Print progress
            $this->verbose(sprintf("Downloaded %d of %d bytes (%.2f%%)\n" , $end + 1 , $fileSize , (($end + 1) / $fileSize) * 100));
        }
    }

    public function SplitVideoCPU($strVideoFile, $intStart, $intDuration, $strOutput){
        try {
            $this->_ffmpeg->open($strVideoFile);
            $startTime = \FFMpeg\Coordinate\TimeCode::fromSeconds($intStart);
            $duration = \FFMpeg\Coordinate\TimeCode::fromSeconds($intDuration);
            $additionalParameters = [
                '-y',
                '-vsync', '0',
                '-i', $strVideoFile,
                '-ss', (string)$startTime,
                '-t', (string)$duration,
                '-threads', '12',
                '-c', 'copy',  // Copy the streams without re-encoding
                $strOutput
            ];
            $this->_ffmpeg->getFFMpegDriver()->command($additionalParameters);
            return true;                        
        }catch(Exception $e){
            $this->verbose($e->getMessage());
            return false;
        }
    }

    public function SplitVideoGPU($strVideoFile, $intStart, $intDuration, $strOutput){
        try {
            $this->_ffmpeg->open($strVideoFile);
            $startTime = \FFMpeg\Coordinate\TimeCode::fromSeconds($intStart);
            $duration = \FFMpeg\Coordinate\TimeCode::fromSeconds($intDuration);
            $additionalParameters = [
                '-y',  // Overwrite output files without asking
                '-vsync','0',  // Overwrite output files without asking
                '-hwaccel', 'cuda',
                '-hwaccel_output_format', 'cuda',
                '-i', $strVideoFile,  // Input file
                '-ss', (string)$startTime,
                '-t', (string)$duration,
                '-threads', '12',
                '-c:v', 'h264_nvenc',
                $strOutput  // Output file
            ];

            $this->_ffmpeg->getFFMpegDriver()->command($additionalParameters);
            return true;                        
        
           
        }catch(Exception $e){
            $this->verbose($e->getMessage());
            return false;
        }
    }

    private function createConcatList($arrFiles){
        if(empty($arrFiles)){
            $this->verbose("No Files");
            return false;
        }
        $strFileList = $this->_tempPath . "ffmpeg-concat-list";
        $strListContent = '';
        $i = 0;
        foreach ($arrFiles as $file) {
            $i++;
            if (!file_exists($file)) {
                $this->verbose("File: $file not exist");
                continue;
            }
            if (!is_readable($file)) {
                $this->verbose("File: $file not readable");
                continue;
            }
            $strListContent .= "file '$file'\n";
        }
        file_put_contents($strFileList, $strListContent);
        return $strFileList;
    }

    private function deleteConcatList($arrFiles, $strFileList){
        foreach($arrFiles as $strFile){
            $this->verbose("Deleting File: $strFile");
            unlink($strFile);
        }
        $this->verbose("Files Deleted, Deleting $strFileList") ;
        unlink($strFileList);
    }

    public function MergeCPU($arrFiles, $strOutput){
        try {
            $this->_message = "Merging files now, it may take a while please be patient ...";
            $this->progress(99, 100);    
            $strFileList = $this->createConcatList($arrFiles);
            $additionalParameters = [
                '-f', 'concat',
                '-safe', '0',
                '-i', $strFileList,
                '-c:v', 'libx264', // Use NVENC for GPU accelerationh264_nvenc
                '-b:v', '5M', // Set a bitrate for NVENC
                '-preset', 'slow', // You can adjust this based on your requirements
                '-c:a', 'aac',
                '-b:a', '192k',
                '-y',
                $strOutput
            ];        
            $this->_ffmpeg->getFFMpegDriver()->command($additionalParameters);
            $this->deleteConcatList($arrFiles, $strFileList);
            return true;
        } catch (\Alchemy\BinaryDriver\Exception\ExecutionFailureException $e) {
            $error = "FFmpeg failed to execute command: " . $e->getMessage();
            $this->verbose($error);
            return false;
        }
    }

    public function MergeGPU($arrFiles, $strOutput){
        try {
            $this->_message = "Merging files now, it may take a while please be patient ...";
            $this->progress(99, 100);    
            $strFileList = $this->createConcatList($arrFiles);
            $additionalParameters = [
                '-f', 'concat',
                '-hwaccel', 'cuda',
                '-safe', '0',
                '-i', $strFileList,
                '-c:v', 'h264_nvenc', // Use NVENC for GPU accelerationh264_nvenc
                '-b:v', '5M', // Set a bitrate for NVENC
                '-preset', 'slow', // You can adjust this based on your requirements
                '-c:a', 'aac',
                '-b:a', '192k',
                '-y',
                $strOutput
            ];        
            $this->_ffmpeg->getFFMpegDriver()->command($additionalParameters);
            $this->deleteConcatList($arrFiles, $strFileList);
            return true;
        } catch (\Alchemy\BinaryDriver\Exception\ExecutionFailureException $e) {
            $error = "FFmpeg failed to execute command: " . $e->getMessage();
            $this->verbose($error);
            return false;
        }
    }

    

}

