<?php
namespace NsMAIN;

class ClsCtrlCronMain extends \NsFWK\ClsCtrlCron {

    protected $_jobId = '';

    protected function ReportProgress($strTitle, $strMessage, $intUserID, $decProgress, $strMessageType,$arrMeta, $tmStartTime = ''){
        $arrJob = $this->prepareJobData($strTitle, $strMessage, $intUserID, $decProgress, $strMessageType,$arrMeta, $tmStartTime);
        $this->updateRedisProgress($arrJob);
    }

    protected function createTmpFolder($strPath){
        if (!is_dir($strPath)) {
            mkdir($strPath, 0755, true);
        }
    }

    protected function deleteDirectory($dir) {
        return is_dir($dir) ? $this->deleteDirectoryContents($dir) && rmdir($dir) : unlink($dir);
    }

    protected function Cleanup(){
        $obj = new \NsFWK\ClsBllRedis();
        $rslt = $obj->DeleteKey($this->_jobId);
        $this->verbose("Deleted Redis Data for Job ID: {$this->_jobId}, Result: $rslt");
        return $rslt;
    }

    protected static function removeDirectory($directory, $deleteParent = null){
        $files = glob($directory . '/{,.}[!.,!..]*', GLOB_MARK | GLOB_BRACE);
        foreach ($files as $file) {
            is_dir($file) ? self::removeDirectory($file, true) : unlink($file);
        }
        if ($deleteParent) {
            rmdir($directory);
        }
    }

    protected function IsAnotherBackgroundJobRunWithSameMatch() {
        $objSrcRedis = new \NsFWK\ClsBllRedis();
        $arrSrc = $objSrcRedis->GetValue($this->_jobId);
        if(!isset($arrSrc['meta']['match_id'])){
            return false;
        }

        $intSrcMatchID = (int)$arrSrc['meta']['match_id'];

        $objRedis = new \NsFWK\ClsBllRedis();
        $arrProcesses = $objRedis->List("*");
        foreach($arrProcesses as $strKeyID){
            $arrEntry = $objRedis->GetValue($strKeyID);
            if(!isset($arrEntry['meta']['match_id'])){
                continue;
            }
            $intMatchID = (int)$arrEntry['meta']['match_id']; 
            if($intMatchID==$intSrcMatchID){
                return true;
            }
        }
        return false;
    }


    private function prepareJobData($strTitle, $strMessage, $intUserID, $decProgress, $strMessageType,$arrMeta, $tmStartTime = ''){
        return [
            'id' => $this->_jobId,
            'progress' => $decProgress,
            'message' => $strMessage,
            'type' => $strMessageType,
            'userId' => $intUserID,
            'title' => $strTitle,
            'consumer' => \NsFWK\ClsHlpHelper::GetServerIP(),
            'meta'=>$arrMeta
        ];
    }

    private function updateRedisProgress($arrJob) {
        $obj = new \NsFWK\ClsBllRedis();
        $rslt = $obj->SetValue($this->_jobId, $arrJob);
        $this->verbose("Redis Set Progress Result: $rslt");
        $this->flushOutput();
    }

    private function deleteDirectoryContents($dir) {
        foreach (scandir($dir) as $item) {
            if ($item !== '.' && $item !== '..') {
                $this->deleteDirectory($dir . DIRECTORY_SEPARATOR . $item);
            }
        }
        return true;
    }
}
