<?php
namespace NsMAIN;

class ClsCtrlPageIndex extends \NsMAIN\ClsCtrlPageCommon {
    protected function do_Default()
    {
        
    }
    // protected function before_Stream(){
    //     if(!isset($this->_data['file'])){
    //         header("HTTP/1.1 400 Bad Request");
    //         echo "File parameter is missing.";
    //         return false;
    //     }

    //     return true;
    // }


    protected function do_Stream() {
        // Define the directory path for video recordings
        $directory = VIDEO__RECORDING__PATH;
    
        // Extract the file name from the provided data
        $file = basename($this->_data['file']);
    
        // Determine the file path based on the mode (temporary or normal)
        if (isset($_GET['mode']) && $_GET['mode'] === 'tmp') {
            $folder = "tmp_" . $this->removeNumberPartFromFile($file);
            $filePath = $directory . DIRECTORY_SEPARATOR . $folder . DIRECTORY_SEPARATOR . $file;
        } else {
            $filePath = $directory . DIRECTORY_SEPARATOR . $file;
        }
    
        // Check if the file exists
        if (!file_exists($filePath)) {
            header("HTTP/1.1 404 Not Found");
            echo "File not found: $filePath";
            exit;
        }
    
        $logFile = LOCAL__TMP . DIRECTORY_SEPARATOR . "test_log.log";
        $this->logHttpRequest($logFile);

        $fileSize = filesize($filePath);
        // Get the HTTP range header if present
        $range = $_SERVER['HTTP_RANGE'] ?? null;

        // Handle HTTP Range requests for seeking
        if ($range) {
            // Parse the Range header
            preg_match('/bytes=(\d*)-(\d*)/', $range, $matches);
            $start = $matches[1] !== '' ? (int)$matches[1] : 0;
            $end = $matches[2] !== '' ? (int)$matches[2] : $fileSize - 1;

            // Validate the range
            if ($start >= $fileSize || $end >= $fileSize || $start > $end) {
                header("HTTP/1.1 416 Requested Range Not Satisfiable");
                header("Content-Range: bytes */$fileSize");
                exit;
            }

            // Set headers for partial content
            $chunkSize = $end - $start + 1;
            header("HTTP/1.1 206 Partial Content");
            header("Content-Range: bytes $start-$end/$fileSize");
            header("Accept-Ranges: bytes");
            header("Content-Length: $chunkSize");
            header("Content-Type: video/mp4"); // Adjust based on the actual video type

            // Open the file and stream the requested range
            $file = fopen($filePath, 'rb');
            fseek($file, $start);
            $bufferSize = 8192; // 8KB buffer size for reading

            while (!feof($file) && ($pos = ftell($file)) <= $end) {
                if ($pos + $bufferSize > $end) {
                    $bufferSize = $end - $pos + 1;
                }
                echo fread($file, $bufferSize);
                flush();
                if (connection_aborted()) break;
            }

            fclose($file);
        } else {
            // Serve the full file if no range is requested
            header("HTTP/1.1 200 OK");
            header("Content-Length: $fileSize");
            header("Content-Type: video/mp4"); // Adjust based on the actual video type

            // Open and stream the entire file
            $file = fopen($filePath, 'rb');
            while (!feof($file)) {
                echo fread($file, 8192);
                flush();
                if (connection_aborted()) break;
            }
            fclose($file);
        }

    }
    


    private function logHttpRequest($logFilePath)
    {
        $logData = [
            'Date' => date('Y-m-d H:i:s'),
            'GET' => $_GET,
            'POST' => $_POST,
            'SERVER_RAW' => $_SERVER,
        ];

    
        // Write log data to the file
        file_put_contents($logFilePath, print_r($logData, true),FILE_APPEND);
    }


    protected function before_Meta(){

        if (!isset($this->_data['file'])) {
            header("HTTP/1.1 400 Bad Request");
            echo "File parameter is missing.";
            return false;
        }
        return true;

    }
    protected function do_Meta(){
        
        $file = basename($this->_data['file']);

        $filepath = VIDEO__RECORDING__PATH . DIRECTORY_SEPARATOR . $file;

        if(!file_exists($filepath)){
            if (preg_match('/_(\d+)\.ts$/', $file, $matches)) {
                // Extract the numeric part and create a directory name from it
                $directorySuffix = $matches[0];
                $directorTmp = "tmp_" . str_replace($directorySuffix, "", $file);
                $directory = VIDEO__RECORDING__PATH . DIRECTORY_SEPARATOR . $directorTmp;
                $filepath = $directory . DIRECTORY_SEPARATOR . $file;
            }
        }

        
        if (!file_exists($filepath)) {
            header("HTTP/1.1 404 Not Found");
            echo "File not found. $filepath";
            return false;
        }
        
        $cmd = MEDIAINFO . " " .  escapeshellarg($filepath);

        $output = shell_exec($cmd);
        if (empty($output)) {
            $arrResult = [];
            $arrResult['result'] = false;
            $arrResult['title'] = 'Error';
            $arrResult['message'] = 'Failed to retrieve video details';
            print json_encode($arrResult);
            return false;
        }
        
        $arrOutput = $this->parseMediaInfo($output);        
        
        header('Content-Type: application/json');
        
        $arrResult = [];
        $arrResult['result'] = true;
        $arrResult['title'] = 'success';
        $arrResult['message'] = 'Video information successfully retrieved';
        $arrResult['object'] = $arrOutput;
        print json_encode($arrResult);
    }


    
    
    private function parseMediaInfo($inputString) {
        $parsedArray = [];
        $currentSection = null;
    
        $lines = explode("\n", $inputString);
    
        foreach ($lines as $line) {
            $line = trim($line);
    
            if (empty($line)) {
                continue;
            }
    
            if (strpos($line, ':') === false) {
                $currentSection = $line;
                $parsedArray[$currentSection] = [];
            } else {
                list($key, $value) = explode(':', $line, 2);
                $key = trim($key);
                $value = trim($value);
    
                // Convert duration to seconds if the key is "Duration"
                if ($key === 'Duration') {
                    $value = $this->convertDurationToSeconds($value);
                }
                if(strtoupper($key)==='FRAME RATE MODE'){
                    if($value=='Variable'){
                        $key = 'fps';
                        $value = 25;
                    }
                }
                if (strtoupper($key) === 'FRAME RATE'){
                    $key = 'fps';
                    $value = (double)trim(str_replace("FPS","",$value));
                }
    
                $parsedArray[$currentSection][$key] = $value;
            }
        }
    
        return $parsedArray;
    }
    

    
    private function removeNumberPartFromFile($filename) {
        $newFilename = preg_replace('/_\d+\.ts$/', '', $filename);
        return $newFilename;
    }
    private function convertDurationToSeconds($duration) {
        $parts = explode(' ', $duration);
        $seconds = 0;
    
        for ($i = 0; $i < count($parts); $i += 2) {
            $value = (int) $parts[$i];
            $unit = $parts[$i + 1];
    
            if (strpos($unit, 'min') !== false) {
                $seconds += $value * 60;
            } elseif (strpos($unit, 'h') !== false) {
                $seconds += $value * 3600;
            } elseif (strpos($unit, 's') !== false) {
                $seconds += $value;
            }
        }
    
        return $seconds;
    }

}